﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Excel
{
    class Funciones
    {

        //return the date in the same excel format
        public double OaDate(int dia, int mes, int year){
            DateTime value = new DateTime(year, mes, dia);
            return (double) value.ToOADate();
        }

        //tooltor

        private int _daysPerYear(int year, int basis)
	    {
            int daysPerYear = 0;
		    switch (basis) {
			    case 0 :
			    case 2 :
			    case 4 :
				    daysPerYear = 360;
				    break;
			    case 3 :
				   daysPerYear = 365;
				    break;
			    case 1 :
				    daysPerYear = (_isLeapYear(year)) ? 366 : 365;
				    break;
		    }
		    return daysPerYear;
	    }

        private bool _isLeapYear(int year) {
		    return (((year % 4) == 0) && ((year % 100) != 0) || ((year % 400) == 0));
	    }

        //is the last day of the month?
        private bool _lastDayOfMonth(DateTime testDate)
        {
            bool eom=false;
            DateTime fecha2 = new DateTime(testDate.Year, testDate.Month + 1, 1).AddDays(-1);
            if(fecha2.Day==testDate.Day)
                eom=true;
            return eom;
        }

        private DateTime _coupFirstPeriodDate(DateTime settlement, DateTime maturity, double frequency, bool next)
        {
		    int months = Int32.Parse((12.00 / frequency).ToString());
            
		    while (settlement < maturity) {
                maturity = maturity.AddMonths(-months);
		    }
		    if (next) {
                maturity=maturity.AddMonths(months);
		    }

		    return maturity;
	    }

        public double COUPDAYS(DateTime settlement, DateTime maturity, double frequency, int basis)
        {
		    
		    switch (basis) {
			    case 3: // Actual/365
					    return 365.00 / frequency;
			    case 1: // Actual/actual
					    if (frequency == 1) {
						    double daysPerYear = _daysPerYear(maturity.Year,basis);
						    return (daysPerYear / frequency);
					    } else {
						    DateTime prev = _coupFirstPeriodDate(settlement, maturity, frequency, false);
						    DateTime next = _coupFirstPeriodDate(settlement, maturity, frequency, true);
                            return (OaDate(next.Day, next.Month, next.Year) - OaDate(prev.Day, prev.Month, prev.Year));
					    }
			    default: // US (NASD) 30/360, Actual/360 or European 30/360
                        return 360.00 / frequency;
		    }
	    }

        private double _dateDiff360(int startDay, int startMonth, int startYear, int endDay, int endMonth, int endYear, int methodUS) {
		    if (startDay == 31) {
			    startDay--;
		    } else if(methodUS==1 && (startMonth == 2 && (startDay == 29 || (startDay == 28 && !_isLeapYear(startYear))))) {
			    startDay = 30;
		    }
		    if (endDay == 31) {
			    if (methodUS==1 && startDay != 30) {
				    endDay = 1;
				    if (endMonth == 12) {
					    ++endYear;
					    endMonth = 1;
				    } else {
					    ++endMonth;
				    }
			    } else {
				    endDay = 30;
			    }
		    }

            return endDay + endMonth * 30.00 + endYear * 360.00 - startDay - startMonth * 30.00 - startYear * 360.00;
	    }

        public double DAYS360(DateTime startDate, DateTime endDate, int method) {
		    
		    // Execute function
		    int startDay = startDate.Day;
		    int startMonth = startDate.Month;
		    int startYear = startDate.Year;

		    int endDay = endDate.Day;
		    int endMonth = endDate.Month;
		    int endYear = endDate.Year;
            
            if(method==0)
                method=1;
            else if(method==1)
                method=0;
            return _dateDiff360(startDay, startMonth, startYear, endDay, endMonth, endYear, method);
	    }

        public double DATEDIF(DateTime startDate, DateTime endDate, String unit) {
		    
		    // Execute function
		    TimeSpan difference = endDate - startDate;

		    int startDays = startDate.Day;
		    int startMonths = startDate.Month;
		    int startYears = startDate.Year;

		    int endDays = endDate.Day;
		    int endMonths = endDate.Month;
		    int endYears = endDate.Year;

		    int retVal = 0;
		    switch (unit) {
			    case "D":
				    retVal = difference.Days;
				    break;
			    case "M":
				    retVal = (endMonths - startMonths) + ((endYears - startYears) * 12);
				    //	We're only interested in full months
				    if (endDays < startDays) {
					    retVal--;
				    }
				    break;
			    case "Y":
				    retVal = (endYears - startYears);
				    //	We're only interested in full months
				    if (endMonths < startMonths) {
					    retVal--;
				    } else if ((endMonths == startMonths) && (endDays < startDays)) {
					    retVal--;
				    }
				    break;
			    case "MD":
				    if (endDays < startDays) {
					    retVal = endDays;
                        endDate=endDate.AddDays(-endDays);
					    int adjustDays = endDate.Day;
					    if (adjustDays > startDays) {
						    retVal += (adjustDays - startDays);
					    }
				    } else {
					    retVal = endDays - startDays;
				    }
				    break;
			    case "YM":
				    retVal = (endMonths - startMonths);
				    if (retVal < 0) retVal = 12 + retVal;
				    //	We're only interested in full months
				    if (endDays < startDays) {
					    retVal--;
				    }
				    break;
			    case "YD":
				    retVal = difference.Days;
				    if (endYears > startYears) {
					    while (endYears > startYears) {
                            endDate = endDate.AddYears(-1);
						    endYears = endDate.Year;
					    }
                        TimeSpan difference2 = endDate - startDate;
					    retVal = difference2.Days;
					    if (retVal < 0) { retVal += 365; }
				    }
				    break;
		    }
		    return retVal;
	    }

        public double YEARFRAC(DateTime startDate, DateTime endDate, int method) {
			    switch(method) {
                    case 0:
                        return DAYS360(startDate, endDate, 0) / 360.00;
				    case 1	:
					    double days = DATEDIF(startDate,endDate,"D");
					    int startYear = startDate.Year;
					    int endYear = endDate.Year;
					    double years = endYear - startYear + 1;
					    double leapDays = 0;
					    if (years == 1) {
						    if (_isLeapYear(endYear)) {
							    int startMonth = startDate.Month;
							    int endMonth = endDate.Month;
							    int endDay = endDate.Day;
							    if ((startMonth < 3) ||
                                    ((endMonth * 100.00 + endDay) >= (2.00 * 100.00 + 29.00)))
                                {
                                    leapDays += 1.00;
							    }
						    }
					    } else {
						    for(int year = startYear; year <= endYear; year++) {
							    if (year == startYear) {
								    int startMonth = startDate.Month;
								    int startDay = startDate.Day;
								    if (startMonth < 3) {
                                        leapDays += _isLeapYear(year) ? 1.00 : 0.00;
								    }
							    } else if(year == endYear) {
								    int endMonth = endDate.Month;
								    int endDay = endDate.Day;
                                    if ((endMonth * 100 + endDay) >= (2.00 * 100.00 + 29.00))
                                    {
                                        leapDays += _isLeapYear(year) ? 1.00 : 0.00;
								    }
							    } else {
                                    leapDays += _isLeapYear(year) ? 1.00 : 0.00;
							    }
						    }
                            if (years == 2.00)
                            {
                                if ((leapDays == 0.00) && (_isLeapYear(startYear)) && (days > 365.00))
                                {
                                    leapDays = 1.00;
                                }
                                else if (days < 366.00)
                                {
                                    years = 1.00;
							    }
						    }
						    leapDays /= years;
					    }
                        return days / (365.00 + leapDays);
				    case 2	:
                        return DATEDIF(startDate, endDate, "D") / 360.00;
				    case 3	:
                        return DATEDIF(startDate, endDate, "D") / 365.00;
				    case 4	:
                        return DAYS360(startDate, endDate, 1) / 360.00;
                    default:
                        return 0;
			    }
	    }

        public double COUPNUM(DateTime settlement, DateTime maturity, double tfrequency, int basis)
        {
            int frequency = Int32.Parse(tfrequency.ToString());
            settlement = _coupFirstPeriodDate(settlement, maturity, tfrequency, true);
            double daysBetweenSettlementAndMaturity = YEARFRAC(settlement, maturity, basis) * 365.00;
            //if daysBetweenSettlementAndMaturity==0, daysBetweenSettlementAndMaturity/perido always will be 0, but need to be 1
            daysBetweenSettlementAndMaturity = daysBetweenSettlementAndMaturity == 0 ? 1 : daysBetweenSettlementAndMaturity;
		    switch (frequency) {
			    case 1: // annual payments
                    return Math.Ceiling(daysBetweenSettlementAndMaturity / 360.00);
			    case 2: // half-yearly
                    return Math.Ceiling(daysBetweenSettlementAndMaturity / 180.00);
			    case 4: // quarterly
                    return Math.Ceiling(daysBetweenSettlementAndMaturity / 90.00);
			    case 6: // bimonthly
                    return Math.Ceiling(daysBetweenSettlementAndMaturity / 60.00);
			    case 12: // monthly
                    return Math.Ceiling(daysBetweenSettlementAndMaturity / 30.00);
                default:
                    return 1.00;
		    }
	    }

        public double COUPDAYBS(DateTime settlement, DateTime maturity, double frequency, int basis) {
		    
		    int daysPerYear = _daysPerYear(settlement.Year,basis);
		    DateTime prev = _coupFirstPeriodDate(settlement, maturity, frequency, false);
            return YEARFRAC(prev, settlement, basis) * daysPerYear;
	    }	

        public double COUPDAYSNC(DateTime settlement, DateTime maturity, double frequency, int basis) {
		    
		    int daysPerYear = _daysPerYear(settlement.Year,basis);
		    DateTime next = _coupFirstPeriodDate(settlement, maturity, frequency, true);
            return YEARFRAC(settlement, next, basis) * daysPerYear;
	    }

    }
}
